function [Ar,Br,Cr,Dr,Er,abserr,hksv] = bt(A,B,C,D,E,mode,tau); 
% BT
% Compute a reduced-order model of a given linear, continuous-time system
% using balanced truncation based on spectral projection.
% 
% INPUT ARGUMENTS:
%    A  - an n x n - matrix.  Note that A or (A,E) if E is nonempty) is
%         assumed to be stable - this is not checked in the code!  An
%         unstable A-matrix will usually cause the sign function-based
%         Lyapunov solver to exit with an error due to failure to converge.
%    B  - an n x m - matrix.
%    C  - an p x n - matrix.
%    D  - an p x m - matrix (D is allowed to be empty).
%    E  - a nonsingular n x n - matrix (E is allowed to be empty, then E is 
%         taken as identity matrix of order n).
%  mode - a vector with 2 entries.  The first entry selects whether the 
%         reduced-order model is computed based on an accuracy threshold 
%         tol (mode(1)=1) or a fixed size r (mode(1)~=1). The second entry 
%         selects either the SR (mode(2)='1') or BFSR (mode(2)~=1) method
%         for computing the reduced-order model from the Gramians. 
%         If neglected, an accuracy threshold of 1e-2 with the SR
%         method is employed.
%    r  - desired order of the reduced-order model.  If r <= 0, a
%         reducded-order model is computed based on an absolute error less
%         than tol, where tol is either the 3rd input argument or set by 
%         default to 1e-4.  
%   tau - absolute error tolerance for the reduced-order model if mode(1)=1 
%         or th edesired order of the reduced-order model depending on 
%         mode(1).  
%
% OUTPUT ARGUMENTS:
%   Ar,Br,Cr,Dr,Er - system matrices of the reduced-order model.  If D/E is 
%                    empty on input, Dr/Er are empty on output. 
%   abserr - optional output argument, contains computed error bound for
%            reduced-order model.
%   hksv   - an optional output argument containing the computed Hankel
%            singular values.
%
% REFERENCE:
%
%   P. BENNER, E.S. QUINTANA-ORTI. Model reduction based on spectral 
%   projection methods.  In: P. Benner, V.L. Mehrmann, D.C. Sorensen
%   (eds.), "Dimension Redution of Large-Scale Systems", vol. 45 of
%   Lecture Notes in Computational Science and Engineering, pp. 5-48,
%   Springer-Verlag, Berlin/Heidelberg, 2005.
%
% Peter Benner,  2006-02-10.

error(nargchk(3, 7, nargin))
n = size(A,1);
m = size(B,2);
p = size(C,1);
if nargin <= 3,
    D = [];
    E = [];
end

%check matrix sizes
if size(A,2) ~= n,  
    error('A must be square.'),  
end
if size(B,1) ~= n,  
    error('B must have the same number of rows as A.'),  
end
if size(C,2) ~= n,  
    error('C must have the same number of columns as A.'),  
end
if ~isempty(E),
    if (size(E,1)~=n | size(E,2)~=n)
        error('E must have the same size as A.')
    else
        A = E\A;
        B = E\B;
    end
end

if nargin<=5, 
  tolselect = 1;
  tol       = 1e-2;
  ordselect = 0;
else
    if mode(1) == 1,
        tolselect = 1;              
        if ( (nargin < 7) | ((nargin == 7) & (tau <= 0)) )
            tol = 1e-2;           
        end 
    else
        tolselect = 0;
        if nargin == 7,
            r = floor(tau);
        else
            r = min(n-1,10);
        end            
        ordselect = 1;
    end
end
        
%% Compute full-rank factors of Gramians based on sign function method.
[S,R] = lyapdl_sgn_fac(A,B,C);

%% Compute reduced-order model.
if (nargin > 5 & mode(2) == 1) | (nargin <= 5),
    if tolselect, 
        [Ar,Br,Cr,r,hksv] = srrom(A,B,C,S,R,tolselect,tol);
    else
        [Ar,Br,Cr,r,hksv] = srrom(A,B,C,S,R,tolselect,r);
    end
else
    if tolselect,
        [Ar,Br,Cr,r,hksv] = bfsrrom(A,B,C,S,R,tolselect,tol);
    else
        [Ar,Br,Cr,r,hksv] = bfsrrom(A,B,C,S,R,tolselect,r);
    end
end
Dr = D;
if isempty(E),
    Er = [];
else
    if nargout > 4,
        E = eye(n);
    end
end

if nargout > 1,
    abserr = 2*(sum(hksv(r+1:end)));
end