function [Q,Z,r,it]=diskm(A,B,tol,maxiter)
%DISKM
%
%  Compute the stable deflating subspace of a matrix pencil A - s B
%  having no eigenvalues on the unit circle using the disk function
%  method, i.e., applying the inverse free iteration to A - s B. 
%
%  Input:
%    A, B    : square matrices such that the matrix pencil A - s B has
%              no eigenvalues on the unit circle.
%    tol     : tolerance for stopping the inverse free iteration
%              (default = 10*eps*n).
%    maxiter : maximum number of iterations (default = 60).
%
%  Output:
%    Q, Z   : orthogonal matrices needed for block Schur decomposition of 
%             the matrix pencil A - s B.
%    r      : number of stable generalized eigenvalues.
%    it     : number of required iterations.
%
%  References:
%
%  [1] Z. BAI, J. DEMMEL, M. GU: "Inverse Free Parallel Spectral Divide
%      and Conquer Algorithms for Nonsymmetric Eigenproblems", 
%      Numerische Mathematik 76:279-308, 1997.
%
%  [2] X. SUN, E.S. QUINTANA-ORT: "Spectral division methods for
%      block generalized Schur decompositions; PRISM Working Note #32 (1996).
%      Available from \texttt{http://www-unix.mcs.anl.gov/prism}.
%  
%  Peter Benner,  01-15-1996, 07-31-2002.

error(nargchk(2,4,nargin))
[na,ma] = size(A);
if na ~= ma,  error('Input matrix A must be square.'),  end
n = na;
[nb,mb] =size(B);
if (nb ~= n | mb ~= n),  error('Matrix dimensions must agree.'),  end

if nargin < 4,	maxiter = 100;	end
if nargin < 3,	tol = 10*eps*n;	end

it = 0;

S  = eye(n);
relerr = 1;

AA = A;
BB = B;

%% Inverse free iteration based on [1]
while ( relerr > tol & it < maxiter )
  SS    = S;
  [U,S] = qr([BB; -AA]);
  S  = S(1:n,:);
  AA = U(1:n,n+1:2*n)'*AA;
  BB = U(n+1:2*n,n+1:2*n)'*BB;
  it =  it +  1;  
  relerr = norm(S - SS,1)/norm(S,1);
  disp(sprintf('|| S_%i - S_%i || / || S_%i || = %d', it, it-1, it, relerr))
end

if (it == maxiter & norm(S - SS,1) > tol*norm(S,1)),
  error(disp(sprintf('DISC: No convergence in %d iterations.', maxiter)))
end

%% Subspace extraction scheme based on [2]
%next two lines should be replace by true RRQR!
[U,R,P] = qr(AA');
r       = rank(R);
[Q,R,P] = qr([A*U(:,r+1:n), B*U(:,r+1:n)]); 
Z = [ U(:,r+1:n), U(:,1:r)];
Q = Q';
r = n-r;